from src.ai_playing.montecarlo import MonteCarlo


def montecarlo_benchmark(n, move_limit=None):
    exec_times, scores, all_moves = 0, 0, 0
    for i in range(n):
        exec_time, score, moves = MonteCarlo().benchmark(move_limit)
        print(
            f"{i + 1} :: Execution Time: {round(exec_time, 2)} s :: Score: {score} :: Moves: {moves}"
        )
        exec_times += exec_time
        scores += score
        all_moves += moves

    print(
        f"RESULT :: Mean Execution Time: {round(exec_times / n, 2)} s "
        f":: Mean Score: {scores // n} :: Mean Moves: {all_moves // n}"
    )


if __name__ == "__main__":
    print("Montecarlo Time Benchmark")
    montecarlo_benchmark(20, 2500)
    print("--------")
    print("Montecarlo Score Benchmark")
    montecarlo_benchmark(30)
    print("--------")



from rich_menu import Menu

from src.ai_playing.montecarlo import MonteCarlo
from src.game import Game


def main() -> None:
    menu = Menu(
        "Single Player",
        "AI Playing",
        "Exit",
        rule_title="2048",
    )
    while True:
        match menu.ask():
            case "Single Player":
                Game().run()
            case "AI Playing":
                MonteCarlo().run()
            case "Exit":
                exit()


if __name__ == "__main__":
    main()



import click
from rich.align import Align
from rich.layout import Layout
from rich.rule import Rule
from rich.text import Text

from src.board import Board


def get_click() -> str | None:
    match click.getchar():
        case "\r":
            return "enter"
        case "\x1b[B" | "s" | "S":
            return "down"
        case "\x1b[A" | "w" | "W":
            return "up"
        case "\x1b[D" | "a" | "A":
            return "left"
        case "\x1b[C" | "d" | "D":
            return "right"
        case "\x1b" | "q" | "Q":
            exit()
        case _:
            return None


def print_board(board: Board, score: int, moves: int, ended=False) -> Layout:
    menu = Text()
    menu.append(Text(" ╭──────┬──────┬──────┬──────╮\n"))
    for i in range(4):
        for j in range(4):
            piece = board[i][j]

            match piece:
                case 2 | 4:
                    color = ""
                case 8 | 16:
                    color = "bright_yellow"
                case 32 | 64:
                    color = "orange1"
                case 128 | 256:
                    color = "dark_orange"
                case 512 | 1024:
                    color = "orange_red1"
                case 2048:
                    color = "red"
                case _:
                    color = "dark_red"

            if j == 0:
                menu.append(Text(f" │ ", end=""))
            if piece:
                if piece < 10:
                    menu.append(Text.assemble(("  "), (f"{piece}", color), ("  │ ")))
                elif piece < 100:
                    menu.append(Text.assemble((" "), (f"{piece}", color), ("  │ ")))
                elif piece < 1000:
                    menu.append(Text.assemble((" "), (f"{piece}", color), (" │ ")))
                else:
                    menu.append(Text.assemble((f"{piece}", color), (" │ ")))
            else:
                menu.append(Text(f"     │ ", end=""))
        menu.append(Text("\n"))
        if i < 3:
            menu.append(Text(" ├──────┼──────┼──────┼──────┤\n"))
        else:
            menu.append(Text(" ╰──────┴──────┴──────┴──────╯\n"))

    layout = Layout()
    layout.split_column(
        Layout(Rule("2048"), size=1),
        Layout(name="score", size=2),
        Layout(Align(menu, "center"), size=9),
    )
    layout["score"].split_row(
        Layout(Text(f"\nMoves: {moves}", justify="right")),
        Layout(Text("\n|", justify="center"), size=3),
        Layout(Text(f"\nScore: {score}", justify="left")),
    )
    if ended:
        layout.add_split(
            Layout(Text("You lose! Press any key to continue", justify="center")),
        )

    return layout



from src.game import Game
from src.rust_montecarlo import montecarlo


class MonteCarlo(Game):
    def get_move(self) -> str:
        iterations = 10 + self.board.moves
        depth = 5 + self.board.moves // 100
        return montecarlo(self.board.board, iterations, depth)


if __name__ == "__main__":
    MonteCarlo().run()



from time import sleep, time

import click
from rich.live import Live

from src.board import Board
from src.utils import get_click, print_board


class Game:
    def run(self):
        self.board = Board()

        layout = print_board(self.board.board, self.board.score, self.board.moves)
        with Live(layout, auto_refresh=False, screen=True) as live:
            live.update(layout, refresh=True)
            while True:
                try:
                    direction_move = self.get_move()
                    if direction_move and self.board.move(direction_move):
                        self.board.new_piece()
                        layout = print_board(
                            self.board.board, self.board.score, self.board.moves
                        )
                    elif self.board.verify_end():
                        layout = print_board(
                            self.board.board,
                            self.board.score,
                            self.board.moves,
                            ended=True,
                        )
                        live.update(layout, refresh=True)
                        sleep(0.8)
                        live.update(layout, refresh=True)
                        click.getchar()
                        return

                    live.update(layout, refresh=True)
                except (KeyboardInterrupt, EOFError):
                    exit()

    def benchmark(self, move_limit=None):
        while True:
            start = time()
            score, moves = self._benchmark_run(move_limit)
            end = time()

            if move_limit is None or moves == move_limit:
                return end - start, score, moves

    def _benchmark_run(self, move_limit=None):
        self.board = Board()
        while True:
            if move_limit is not None and move_limit == self.board.moves:
                return self.board.score, self.board.moves
            direction_move = self.get_move()
            if direction_move and self.board.move(direction_move):
                self.board.new_piece()
            elif self.board.verify_end():
                return self.board.score, self.board.moves

    def get_move(self):
        try:
            dir = get_click()
        except (KeyboardInterrupt, EOFError):
            exit()
        if dir != "enter":
            return dir



import random

import numpy as np


class Board:
    def __init__(self) -> None:
        self.new_game()

    def new_game(self) -> None:
        self.score = 0
        self.moves = 0
        self.board = np.array(
            [
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
            ]
        )

        for _ in range(2):
            pos = True
            while pos:
                i, j = random.sample(range(4), 2)
                pos = self.board[i][j]

            if random.randint(1, 10) == 1:
                self.board[i][j] = 4
            else:
                self.board[i][j] = 2

    def move(self, direction: str) -> bool:
        if direction == "down":
            self.board = np.flip(self.board)
            self.board = np.rot90(self.board)
            has_moved = self._moving()
            self.board = np.rot90(self.board, 3)
            self.board = np.flip(self.board)

        elif direction == "up":
            self.board = np.rot90(self.board)
            has_moved = self._moving()
            self.board = np.rot90(self.board, 3)

        elif direction == "right":
            self.board = np.fliplr(self.board)
            has_moved = self._moving()
            self.board = np.fliplr(self.board)

        elif direction == "left":
            has_moved = self._moving()

        if has_moved:
            self.moves += 1

        return has_moved

    def new_piece(self) -> bool:
        empty_positions = []
        for i in range(4):
            for j in range(4):
                if self.board[i][j] == 0:
                    empty_positions.append([i, j])

        if not empty_positions:
            return False

        i, j = random.choice(empty_positions)
        if random.randint(1, 10) == 10:
            self.board[i][j] = 4
        else:
            self.board[i][j] = 2

        return True

    def verify_end(self) -> bool:
        empty_positions = []
        for i in range(4):
            for j in range(4):
                if self.board[i][j] == 0:
                    empty_positions.append([i, j])

        if empty_positions:
            return False

        for i in range(4):
            for j in range(4):
                if j + 1 < 4:
                    if self.board[i][j] == self.board[i][j + 1]:
                        return False
                if j - 1 >= 0:
                    if self.board[i][j] == self.board[i][j - 1]:
                        return False
                if i + 1 < 4:
                    if self.board[i][j] == self.board[i + 1][j]:
                        return False
                if i - 1 >= 0:
                    if self.board[i][j] == self.board[i - 1][j]:
                        return False

        return True

    def _moving(self) -> bool:
        self.has_moved = False
        self._push()
        if self._sum():
            self._push()

        return self.has_moved

    def _push(self) -> None:
        for i in range(4):
            for j in range(1, 4):
                if self.board[i][j] != 0:
                    k = j
                    while k > 0 and self.board[i][k - 1] == 0:
                        self.board[i][k - 1], self.board[i][k] = (
                            self.board[i][k],
                            0,
                        )
                        self.has_moved = True
                        k -= 1

    def _sum(self) -> bool:
        result = False
        for i in range(4):
            for j in range(1, 4):
                if self.board[i][j] != 0:
                    if self.board[i][j - 1] == self.board[i][j]:
                        self.score += self.board[i][j] * 2
                        self.board[i][j - 1], self.board[i][j] = (
                            self.board[i][j] * 2,
                            0,
                        )
                        result = True
                        self.has_moved = True
        return result
